local ADDON_NAME, addonTable = ...
ElkFactions = LibStub("AceAddon-3.0"):NewAddon("ElkFactions", "AceBucket-3.0", "AceConsole-3.0", "AceHook-3.0")

local string_format = string.format
local string_gsub   = string.gsub

local MAX_FACTION_INDEX = 300

local LDB = LibStub("LibDataBroker-1.1")
local LDBIcon = LibStub("LibDBIcon-1.0")
local LibQTip = LibStub("LibQTip-1.0")
local HBD = LibStub("HereBeDragons-1.0", true)

local gender = UnitSex("player")

local reputationSessionBase = {}

local DBdefaults = {
	profile = {
		minimap = {
			hide = false,
		},
		patternChat = "{n}: {s} - {rc}/{rm} ({rp}) [{rd}]",
		patternText = "{cw}{n}{cr}: {cb}{s}{cr} - {ci}{rc}/{rm} ({rp}){cr} [{cd}{rd}{cr}]",
		tooltipColorName = "atWar",
		tooltipColorStandingText = "standingBlizzard",
		tooltipColorStandingValueAbsolute = "standingIncremential",
		tooltipColorStandingValuePercent = "standingIncremential",
		tooltipColorStandingValueDelta = "delta",
		tooltipShowHint = false,
		autoZoneFactionEnabled = false,
		AutoZoneFaction = {},
	},
}


-- -----
-- HELPER FUNCTIONS
-- -----

local watchedFactionIndex = nil
local function GetWatchedFactionIndex()
	local watchedID = select(6, GetWatchedFactionInfo())
	if not watchedID or watchedID == 0 then
		-- no faction is currently watched
		watchedFactionIndex = 0
		return 0
	end
	if watchedFactionIndex and select(14, GetFactionInfo(watchedFactionIndex)) == watchedID then
		-- we have an index cached and it's still correct
		return watchedFactionIndex
	end
	
	local factionIndex = 1
	repeat
		if select(12, GetFactionInfo(factionIndex)) then
			watchedFactionIndex = factionIndex
			return watchedFactionIndex
		end
		factionIndex = factionIndex + 1
	until factionIndex > MAX_FACTION_INDEX
	return 0
end

local function GetFactionIndexById(factionIdLookup)
	local factionIndex = 1
	local factionIdPrevious = nil
	repeat
		local factionId = select(14, GetFactionInfo(factionIndex))
		if factionId == nil and factionIdPrevious == nil then break end
		if factionIdLookup == factionId then
			return factionIndex
		end
		factionIdPrevious = factionId
		factionIndex = factionIndex + 1
	until factionIndex > MAX_FACTION_INDEX
	return 0
end

local function SetWatchedFaction(factionId)
--	self:Print("SetWatchedFaction(", factionId, ") called...")
	if not factionId then
		SetWatchedFactionIndex(0)
		return
	end
	local factionIndex = GetFactionIndexById(factionId)
	if factionIndex then
		SetWatchedFactionIndex(factionIndex)
	end
end

-- -----
local function do_OnEnter(frame)
	local tooltip = LibQTip:Acquire("ElkFactionsTip")
	tooltip:SmartAnchorTo(frame)
	tooltip:SetAutoHideDelay(0.1, frame)
	tooltip:EnableMouse(true)
	ElkFactions:UpdateTooltip()
	tooltip:Show()
end

local function do_OnLeave()
	-- empty dummy
end

local function do_OnClick(frame, button)
	if button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory("ElkFactions")
	else
		if IsShiftKeyDown() then
			local watchedIndex = GetWatchedFactionIndex()
			if watchedIndex then
				local activeWindow = ChatEdit_GetActiveWindow()
				if activeWindow then
					local text = ElkFactions:GetFactionText(ElkFactions.db.profile.patternChat, watchedIndex, false)
					activeWindow:Insert(text)
				end
			end
		else
			ToggleCharacter("ReputationFrame")
		end
	end
end

function ElkFactions:OnInitialize()
	self.dbo = LDB:NewDataObject("ElkFactions", {
			type = "data source",
			text = "",
			icon = [[Interface\Icons\Spell_Holy_SealOfSalvation]],
			OnEnter = do_OnEnter,
			OnLeave = do_OnLeave,
			OnClick = do_OnClick,
		})

	self.db = LibStub("AceDB-3.0"):New("ElkFactionsDB", DBdefaults, true)
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")

	LDBIcon:Register("ElkFactions", self.dbo, self.db.profile.minimap)
	
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ElkFactions", "ElkFactions")
end

function ElkFactions:OnEnable()
	-- events for faction data changes
	self:RegisterBucketEvent({"UPDATE_FACTION", "PLAYER_ENTERING_WORLD"}, .1, "FactionUpdate")

	-- trying to fix ticket #9
	self:RegisterBucketEvent("PLAYER_GUILD_UPDATE", .1)

	-- events for zone changes
	if HBD then
--		self:RegisterBucketEvent({"ZONE_CHANGED", "ZONE_CHANGED_INDOORS", "ZONE_CHANGED_NEW_AREA", "MINIMAP_ZONE_CHANGED", "PLAYER_ENTERING_WORLD"}, .1, "ZoneUpdate")
		HBD.RegisterCallback(self, "PlayerZoneChanged", "ApplyAutoZoneFaction")
	end

	-- hook UI functions to be informed of some user actions
	self:Hook("SetWatchedFactionIndex", "Hook_SetWatchedFactionIndex", true)

	-- refresh session's base reputations
	self:UpdateBaseReputations()
end

function ElkFactions:RefreshConfig()
	-- minimap icon
	LDBIcon:Refresh("ElkFactions", self.db.profile.minimap)
	
	-- auto faction
 	self:ApplyAutoZoneFaction()
	
	-- LDB text
	self:UpdateText()
	
	-- tooltip
	self:UpdateTooltip()
end

-- trying to fix ticket #9
function ElkFactions:PLAYER_GUILD_UPDATE(unit)
	if unit and unit ~= "player" then return end
	self:FactionUpdate()
end

function ElkFactions:FactionUpdate()
	self:ApplyAutoZoneFaction()
		
	-- refresh session's base reputations
	self:UpdateBaseReputations()
	
	-- update the LDB text
	self:UpdateText()
	
	-- update the tooltip
	self:UpdateTooltip()
end

function ElkFactions:UpdateBaseReputations()
	local factionIndex = 1
	local namePrevious = nil
	repeat
		local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(factionIndex)
		if name == nil and namePrevious == nil then break end
		if factionID then
			if not reputationSessionBase[factionID] then
				if C_Reputation.IsFactionParagon(factionID) then
					local currentValue, threshold, rewardQuestID, hasRewardPending = C_Reputation.GetFactionParagonInfo(factionID)
					barValue = barValue + currentValue
				end
				reputationSessionBase[factionID] = barValue
			end
		end
		namePrevious = name
		factionIndex = factionIndex + 1
	until factionIndex > MAX_FACTION_INDEX
end

local currentPlayerZoneMapIDCached
function ElkFactions:ZoneUpdate()
	local currentPlayerZoneMapID = HBD:GetPlayerZone() or UNKNOWN
	-- if the current zone changed, refresh the automatic zone faction
	if currentPlayerZoneMapID ~= currentPlayerZoneMapIDCached then
		self:ApplyAutoZoneFaction()
		currentPlayerZoneMapIDCached = currentPlayerZoneMapID
	end
end

function ElkFactions:UpdateText()
	local watchedIndex = GetWatchedFactionIndex()
	if watchedIndex and watchedIndex ~= 0 then
		self.dbo.text = self:GetFactionText(self.db.profile.patternText, watchedIndex, true)
	else
		self.dbo.text = ""
	end
end

local FACTION_COLORS_BLIZZARD = setmetatable({}, {__index = function(self, key) local color = FACTION_BAR_COLORS[key]; local value = color and ("|cff%.2x%.2x%.2x"):format(color.r * 255, color.g * 255, color.b * 255) or nil; rawset(self, key, value); return value end })
local FACTION_COLORS_INC_old = {
	[1] = "|cff8c0000",			-- hated
	[2] = "|cffff0000",			-- hostile
	[3] = "|cffff8c00",			-- unfriendly
	[4] = "|cffbfbfbf",			-- neutral
	[5] = "|cffffffff",			-- friendly
	[6] = "|cff00ff00",			-- honored
	[7] = "|cff3f66e5",			-- reverted
	[8] = "|cff9933cc",			-- exalted
}
local FACTION_COLORS_INC = {
	[1] = "|cffff0000",			-- hated
	[2] = "|cffff8c00",			-- hostile
	[3] = "|cffffff00",			-- unfriendly
	[4] = "|cffffffff",			-- neutral
	[5] = "|cff00ff00",			-- friendly
	[6] = "|cff3f66e5",			-- honored
	[7] = "|cff9933cc",			-- reverted
	[8] = "|cffe6cc80",			-- exalted
}

local function GetFactionDeltaText(factionDelta)
	return (factionDelta == 0  and "-") or (factionDelta > 0 and "+"..factionDelta) or factionDelta
end

local function GetFactionDeltaColor(factionDelta)
	return (factionDelta == 0 and "|cffffffff") or (factionDelta > 0 and "|cff00ff00") or ("|cffff0000")
end

local function tooltip_line_OnMouseUp(frame, factionIndex, button)
	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(factionIndex)
	if IsControlKeyDown() then
		if isHeader then
			if isCollapsed then
				ExpandFactionHeader(factionIndex)
			else
				CollapseFactionHeader(factionIndex)
			end
		else
			if IsFactionInactive(factionIndex) then
				SetFactionActive(factionIndex)
			else
				SetFactionInactive(factionIndex)
			end
		end
	elseif IsShiftKeyDown() then
		local activeWindow = ChatEdit_GetActiveWindow()
		if activeWindow then
			local text = ElkFactions:GetFactionText(ElkFactions.db.profile.patternChat, factionIndex, false)
			activeWindow:Insert(text)
		end
		return
	elseif IsAltKeyDown() and HBD and ElkFactions.db.profile.autoZoneFactionEnabled then
		local watchedID = select(6, GetWatchedFactionInfo())
		local autoZoneFaction = ElkFactions:GetAutoZoneFaction()
		if watchedID and not autoZoneFaction then
			-- we start watching an auto-faction, store the manually watched
			ElkFactions.db.char.lastFaction = watchedID
		end
		local currentPlayerZoneMapID = HBD:GetPlayerZone() or UNKNOWN
		if autoZoneFaction == factionID then
			ElkFactions.db.profile.AutoZoneFaction[currentPlayerZoneMapID] = nil
		else
			ElkFactions.db.profile.AutoZoneFaction[currentPlayerZoneMapID] = factionID
		end
		if not ElkFactions:ApplyAutoZoneFaction() then
			ElkFactions:UpdateTooltip()
		end
	elseif not isHeader or hasRep then
		if factionID == ElkFactions.db.char.lastFaction then
			ElkFactions.db.char.lastFaction = nil
			ElkFactions:UpdateTooltip()
		elseif ElkFactions:GetAutoZoneFaction() then
			ElkFactions.db.char.lastFaction = factionID
			ElkFactions:UpdateTooltip()
		elseif isWatched then
			PlaySound("igMainMenuOptionCheckBoxOff")
			SetWatchedFactionIndex(0)
		else
			PlaySound("igMainMenuOptionCheckBoxOn")
			SetWatchedFactionIndex(factionIndex)
		end
	end
end

local tooltipColors = setmetatable({
	atWar = "",
	delta = "",
	standingBlizzard = "",
	standingIncremential = "",
	standingIncrementialOld = "",
}, {__index = function(self, key) return "" end})
function ElkFactions:UpdateTooltip()
	if not LibQTip:IsAcquired("ElkFactionsTip") then
		return
	end
	local tooltip = LibQTip:Acquire("ElkFactionsTip")
	tooltip:Clear()
	tooltip:SetColumnLayout(7, "CENTER", "LEFT", "LEFT", "CENTER", "CENTER", "RIGHT", "RIGHT")

	local profile = self.db.profile
	local curLine, curColumn

	curLine = tooltip:AddHeader()
	tooltip:SetCell(curLine, 1, "|cfffed100Elk|cffffffffFactions", tooltip:GetHeaderFont(), "CENTER", 7)

-- -----
--~ 	local colortest_inc = "  "
--~ 	local colortest_inc_old = "  "
--~ 	local colortest_blizz = "  "
--~ 	for i = 1, 8 do
--~ 		colortest_inc		= colortest_inc		..FACTION_COLORS_INC[i]			..GetText("FACTION_STANDING_LABEL"..i, gender).."  "
--~ 		colortest_inc_old	= colortest_inc_old	..FACTION_COLORS_INC_old[i]		..GetText("FACTION_STANDING_LABEL"..i, gender).."  "
--~ 		colortest_blizz		= colortest_blizz	..FACTION_COLORS_BLIZZARD[i]	..GetText("FACTION_STANDING_LABEL"..i, gender).."  "
--~ 	end
--~ 	curLine = tooltip:AddLine()
--~ 	tooltip:SetCell(curLine, 1, colortest_inc, nil, "CENTER", 5)
--~ 	curLine = tooltip:AddLine()
--~ 	tooltip:SetCell(curLine, 1, colortest_inc_old, nil, "CENTER", 5)
--~ 	curLine = tooltip:AddLine()
--~ 	tooltip:SetCell(curLine, 1, colortest_blizz, nil, "CENTER", 5)
--~ 	curLine = tooltip:AddLine(" ")
-- -----
	local watchedID = select(6, GetWatchedFactionInfo())
	local autoZoneFaction = self:GetAutoZoneFaction()

	local numFactions = GetNumFactions()
	local indentation
	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus
	for factionIndex=1, numFactions do
		name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(factionIndex)
		if factionID ~= nil then
			local standingText = GetText("FACTION_STANDING_LABEL"..standingID, gender)
			local standingID_color = standingID
			local isCapped = standingID == MAX_REPUTATION_REACTION
			indentation = isChild and "     " or ""
			if isHeader then
				indentation = indentation .. (isCollapsed and "|TInterface\\Buttons\\UI-PlusButton-Up:14:14|t " or "|TInterface\\Buttons\\UI-MinusButton-Up:14:14|t ")
			else
				indentation = indentation .. "     "
			end

			if C_Reputation.IsFactionParagon(factionID) then
				local currentValue, threshold, rewardQuestID, hasRewardPending = C_Reputation.GetFactionParagonInfo(factionID)
				local paragonMin = currentValue - (currentValue % threshold)
				local paragonMax = paragonMin + threshold
				barValue = barValue + currentValue
				barMin = barMin + paragonMin
				barMax = barMax + paragonMax
				isCapped = false
			end
			
			-- check if this is a friendship faction 
			local friendID, friendRep, friendMaxRep, friendName, friendText, friendTexture, friendTextLevel, friendThreshold, nextFriendThreshold = GetFriendshipReputation(factionID)
			if (friendID ~= nil) then
				name = friendName
				standingText = friendTextLevel
				standingID_color = 5
				if ( nextFriendThreshold ) then
					barMin, barMax, barValue = friendThreshold, nextFriendThreshold, friendRep
				else
					barMin, barMax, barValue = 0, 8400, 8400
					isCapped = true
				end
			end

			local repIs = barValue - barMin
			local repMax = barMax - barMin
			local repDelta = not friendID and reputationSessionBase[factionID] and (barValue - reputationSessionBase[factionID]) or 0 --TODO: fix for friends
			
			tooltipColors.atWar = atWarWith and "|cffff0000" or canToggleAtWar and "|cffffff00" or "|cff00ff00"
			tooltipColors.delta = GetFactionDeltaColor(repDelta)
			tooltipColors.standingBlizzard = FACTION_COLORS_BLIZZARD[standingID_color] or "|cfff433ff"
			tooltipColors.standingIncremential = FACTION_COLORS_INC[standingID_color] or "|cfff433ff"
			tooltipColors.standingIncrementialOld = FACTION_COLORS_INC_old[standingID_color] or "|cfff433ff"
			
			if not isHeader or hasRep then
				curLine, curColumn = tooltip:AddLine(), 1
				local icons = ((factionID == (not autoZoneFaction and watchedID or self.db.char.lastFaction)) and "|TInterface\\Buttons\\UI-CheckBox-Check:14:14|t" or "") .. ((factionID == autoZoneFaction) and "|TInterface\\Icons\\Ability_Hunter_SniperShot:14:14|t" or "")
				curLine, curColumn = tooltip:SetCell(curLine, curColumn, icons)
				curLine, curColumn = tooltip:SetCell(curLine, curColumn, tooltipColors[profile.tooltipColorName]                 .. indentation..name, nil, nil, 2)
				curLine, curColumn = tooltip:SetCell(curLine, curColumn, tooltipColors[profile.tooltipColorStandingText]         .. standingText..(hasBonusRepGain and " (+)" or ""))
				if isCapped then
					curLine, curColumn = tooltip:SetCell(curLine, curColumn, tooltipColors[profile.tooltipColorStandingValueAbsolute] .. CAPPED, nil, nil, 2)
				else
					curLine, curColumn = tooltip:SetCell(curLine, curColumn, tooltipColors[profile.tooltipColorStandingValueAbsolute] .. string_format("%5d / %5d", repIs, repMax))
					curLine, curColumn = tooltip:SetCell(curLine, curColumn, tooltipColors[profile.tooltipColorStandingValuePercent]  .. string_format("%.1f%%", repIs / repMax * 100))
				end
				curLine, curColumn = tooltip:SetCell(curLine, curColumn, tooltipColors[profile.tooltipColorStandingValueDelta]   .. GetFactionDeltaText(repDelta))
			else
				curLine, curColumn = tooltip:AddLine(), 2
				curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cffffffff"..indentation..name, nil, nil, 2)
			end
			tooltip:SetLineScript(curLine, "OnMouseUp", tooltip_line_OnMouseUp, factionIndex)
		end
	end
	
	if self.db.profile.tooltipShowHint then
		tooltip:AddSeparator()
		curLine, curColumn = tooltip:AddLine(), 1
		curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100Click:", nil, "LEFT", 2)
		curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100set watched faction", nil, nil, 5)
		if self.db.profile.autoZoneFactionEnabled then
			curLine, curColumn = tooltip:AddLine(), 1
			curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100Alt-Click:", nil, "LEFT", 2)
			curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100set auto-watched faction", nil, nil, 5)
		end 
		curLine, curColumn = tooltip:AddLine(), 1
		curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100Ctrl-Click (faction):", nil, "LEFT", 2)
		curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100set active/inactive", nil, nil, 5)
		curLine, curColumn = tooltip:AddLine(), 1
		curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100Ctrl-Click (header):", nil, "LEFT", 2)
		curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100collapse/expand", nil, nil, 5)
		curLine, curColumn = tooltip:AddLine(), 1
		curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100Shift-Click:", nil, "LEFT", 2)
		curLine, curColumn = tooltip:SetCell(curLine, curColumn, "|cfffed100write to chatbox", nil, nil, 5)
	end
	
	tooltip:UpdateScrolling()
end

function ElkFactions:GetFactionText(pattern, factionIndex, useColor)
	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(factionIndex)
	local standingText = GetText("FACTION_STANDING_LABEL"..standingID, gender)
	local standingID_color = standingID
	local isCapped = standingID == MAX_REPUTATION_REACTION

	if C_Reputation.IsFactionParagon(factionID) then
		local currentValue, threshold, rewardQuestID, hasRewardPending = C_Reputation.GetFactionParagonInfo(factionID)
		local paragonMin = currentValue - (currentValue % threshold)
		local paragonMax = paragonMin + threshold
		barValue = barValue + currentValue
		barMin = barMin + paragonMin
		barMax = barMax + paragonMax
		isCapped = false
	end

	-- check if this is a friendship faction 
	local friendID, friendRep, friendMaxRep, friendName, friendText, friendTexture, friendTextLevel, friendThreshold, nextFriendThreshold = GetFriendshipReputation(factionID)
	if (friendID ~= nil) then
		standingText = friendTextLevel
		standingID_color = 5
		if ( nextFriendThreshold ) then
			barMin, barMax, barValue = friendThreshold, nextFriendThreshold, friendRep
		else
			barMin, barMax, barValue = 0, 8400, 8400
			isCapped = true
		end
	end

	local repIs = barValue - barMin
	local repMax = barMax - barMin
	local repDelta = not friendID and reputationSessionBase[factionID] and (barValue - reputationSessionBase[factionID]) or 0 --TODO: fix for friends
	
	pattern = pattern or ""
	
	--[[
		{n}    name
		{s}    standing
		{rc}   current reputation for this level
		{rm}   maximum reputation for this level
		{rp}   reputation percentage for this level
		{rd}   reputation change this session
		{cw}   color: at war
		{cb}   color: Blizzard
		{ci}   color: incremential
		{cd}   color: reputation change this session
		{cr}   color: reset
	]]
	local color_atwar = atWarWith and "|cffff0000" or canToggleAtWar and "|cffffff00" or "|cff00ff00"
	pattern = string_gsub(pattern, "%b{}", {
		["{n}"] = name,
		["{s}"] = standingText,
		["{rc}"] = isCapped and CAPPED or repIs,
		["{rm}"] = isCapped and "-" or repMax,
		["{rp}"] = isCapped and "-" or string_format("%.1f%%", repIs / repMax * 100),
		["{rd}"] = GetFactionDeltaText(repDelta),
		["{cw}"] = useColor and color_atwar or "",
		["{cb}"] = useColor and (FACTION_COLORS_BLIZZARD[standingID_color] or "|cfff433ff") or "",
		["{ci}"] = useColor and (FACTION_COLORS_INC[standingID_color] or "|cfff433ff") or "",
		["{co}"] = useColor and (FACTION_COLORS_INC_old[standingID_color] or "|cfff433ff") or "",
		["{cd}"] = useColor and GetFactionDeltaColor(repDelta) or "",
		["{cr}"] = useColor and "|r" or "",
	})
	
	return pattern
end

function ElkFactions:GetAutoZoneFaction()
	if not HBD or not self.db.profile.autoZoneFactionEnabled then
		return nil
	end
	
	local currentPlayerZoneMapID = HBD:GetPlayerZone() or UNKNOWN
	return self.db.profile.AutoZoneFaction[currentPlayerZoneMapID]
end

local lastAutoZoneFaction = 0
function ElkFactions:ApplyAutoZoneFaction()
	local watchedID = select(6, GetWatchedFactionInfo())
	local autoZoneFaction = self:GetAutoZoneFaction()
	local changes = false
--[===[@debug@
--	self:Print("ApplyAutoZoneFaction() called...", "watchedID: "..tostring(watchedID), "autoZoneFaction: "..tostring(autoZoneFaction), "lastFaction: "..tostring(self.db.char.lastFaction))
--@end-debug@]===]
	if autoZoneFaction then
		-- there is an auto-faction set for the current zone
		if autoZoneFaction ~= watchedID then
			self.SettingAutoFaction = true
			SetWatchedFaction(autoZoneFaction)
			changes = true
		end
		lastAutoZoneFaction = autoZoneFaction
	elseif self.db.char.lastFaction ~= nil then
		-- no auto-faction set so restore last faction if set
		if self.db.char.lastFaction ~= watchedID then
			self.SettingAutoFaction = true
			SetWatchedFaction(self.db.char.lastFaction)
			changes = true
		end
		self.db.char.lastFaction = nil
		lastAutoZoneFaction = 0
	elseif lastAutoZoneFaction ~= 0 then
		-- no auto-faction and no last faction -> clear watched
		if (watchedID ~= 0) then
			self.SettingAutoFaction = true
			SetWatchedFaction(0)
			changes = true
		end
		lastAutoZoneFaction = 0
	end
	return changes
end

function ElkFactions:Hook_SetWatchedFactionIndex(factionIndex)

--[===[@debug@
--	self:Print("SetWatchedFactionIndex("..tostring(factionIndex)..") called...", "SettingAutoFaction: "..tostring(self.SettingAutoFaction), "watchedIndex: "..tostring(GetWatchedFactionIndex()))
--@end-debug@]===]

	-- store index for lookup
	watchedFactionIndex = factionIndex and factionIndex > 0 and factionIndex or nil

	if self.SettingAutoFaction then
		-- we are setting the auto-faction
		self.SettingAutoFaction = nil
	else
		-- a faction to watch was set; remember and restore auto-faction
		self.db.char.lastFaction = factionIndex and factionIndex > 0 and select(14, GetFactionInfo(factionIndex)) or nil
	end
end
